@@ -8,6 +8,7 @@
 use super::{ArrayBase, ArrayView, Axis, Data, Dimension, NdProducer};
 use crate::aliases::{Ix1, IxDyn};
 use std::fmt;
+use alloc::format;
 
 /// Default threshold, below this element count, we don't ellipsize
 const ARRAY_MANY_ELEMENT_LIMIT: usize = 500;
@@ -287,6 +288,8 @@ where
 mod formatting_with_omit {
     use itertools::Itertools;
     use std::fmt;
+    use alloc::string::String;
+    use alloc::vec::Vec;
 
     use super::*;
     use crate::prelude::*;
