@@ -1,4 +1,4 @@
-// Copyright 2014-2016 bluss and ndarray developers.
+// Copyright 2014-2020 bluss and ndarray developers.
 //
 // Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
 // http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
@@ -12,13 +12,14 @@ use crate::numeric_util;
 use crate::{LinalgScalar, Zip};
 
 use std::any::TypeId;
+use alloc::vec::Vec;
 
 #[cfg(feature = "blas")]
 use std::cmp;
 #[cfg(feature = "blas")]
 use std::mem::swap;
 #[cfg(feature = "blas")]
-use std::os::raw::c_int;
+use libc::c_int;
 
 #[cfg(feature = "blas")]
 use cblas_sys as blas_sys;
@@ -325,9 +326,9 @@ where
 
         // Avoid initializing the memory in vec -- set it during iteration
         unsafe {
-            let mut c = Array::uninitialized(m);
-            general_mat_vec_mul(A::one(), self, rhs, A::zero(), &mut c);
-            c
+            let mut c = Array1::uninit(m);
+            general_mat_vec_mul_impl(A::one(), self, rhs, A::zero(), c.raw_view_mut().cast::<A>());
+            c.assume_init()
         }
     }
 }
@@ -598,6 +599,31 @@ pub fn general_mat_vec_mul<A, S1, S2, S3>(
     S2: Data<Elem = A>,
     S3: DataMut<Elem = A>,
     A: LinalgScalar,
+{
+    unsafe {
+        general_mat_vec_mul_impl(alpha, a, x, beta, y.raw_view_mut())
+    }
+}
+
+/// General matrix-vector multiplication
+///
+/// Use a raw view for the destination vector, so that it can be uninitalized.
+///
+/// ## Safety
+///
+/// The caller must ensure that the raw view is valid for writing.
+/// the destination may be uninitialized iff beta is zero.
+#[allow(clippy::collapsible_else_if)]
+unsafe fn general_mat_vec_mul_impl<A, S1, S2>(
+    alpha: A,
+    a: &ArrayBase<S1, Ix2>,
+    x: &ArrayBase<S2, Ix1>,
+    beta: A,
+    y: RawArrayViewMut<A, Ix1>,
+) where
+    S1: Data<Elem = A>,
+    S2: Data<Elem = A>,
+    A: LinalgScalar,
 {
     let ((m, k), k2) = (a.dim(), x.dim());
     let m2 = y.dim();
@@ -626,22 +652,20 @@ pub fn general_mat_vec_mul<A, S1, S2, S3>(
                         let x_stride = x.strides()[0] as blas_index;
                         let y_stride = y.strides()[0] as blas_index;
 
-                        unsafe {
-                            blas_sys::$gemv(
-                                layout,
-                                a_trans,
-                                m as blas_index,            // m, rows of Op(a)
-                                k as blas_index,            // n, cols of Op(a)
-                                cast_as(&alpha),            // alpha
-                                a.ptr.as_ptr() as *const _, // a
-                                a_stride,                   // lda
-                                x.ptr.as_ptr() as *const _, // x
-                                x_stride,
-                                cast_as(&beta),           // beta
-                                y.ptr.as_ptr() as *mut _, // x
-                                y_stride,
-                            );
-                        }
+                        blas_sys::$gemv(
+                            layout,
+                            a_trans,
+                            m as blas_index,            // m, rows of Op(a)
+                            k as blas_index,            // n, cols of Op(a)
+                            cast_as(&alpha),            // alpha
+                            a.ptr.as_ptr() as *const _, // a
+                            a_stride,                   // lda
+                            x.ptr.as_ptr() as *const _, // x
+                            x_stride,
+                            cast_as(&beta),           // beta
+                            y.ptr.as_ptr() as *mut _, // x
+                            y_stride,
+                        );
                         return;
                     }
                 }
@@ -655,11 +679,12 @@ pub fn general_mat_vec_mul<A, S1, S2, S3>(
         /* general */
 
         if beta.is_zero() {
-            Zip::from(a.outer_iter()).and(y).apply(|row, elt| {
-                *elt = row.dot(x) * alpha;
+            // when beta is zero, c may be uninitialized
+            Zip::from(a.outer_iter()).and(y).for_each(|row, elt| {
+                elt.write(row.dot(x) * alpha);
             });
         } else {
-            Zip::from(a.outer_iter()).and(y).apply(|row, elt| {
+            Zip::from(a.outer_iter()).and(y).for_each(|row, elt| {
                 *elt = *elt * beta + row.dot(x) * alpha;
             });
         }
@@ -683,7 +708,7 @@ fn cast_as<A: 'static + Copy, B: 'static + Copy>(a: &A) -> B {
 #[cfg(feature = "blas")]
 fn blas_compat_1d<A, S>(a: &ArrayBase<S, Ix1>) -> bool
 where
-    S: Data,
+    S: RawData,
     A: 'static,
     S::Elem: 'static,
 {
