@@ -6,12 +6,9 @@
 )]
 
 use ndarray::prelude::*;
-use ndarray::Ix;
-use ndarray::{arr2, arr3, aview1, indices, s, Axis, Data, Dimension, Slice, Zip};
+use ndarray::{arr3, aview1, indices, s, Axis, Slice, Zip};
 
-use itertools::assert_equal;
-use itertools::{enumerate, rev};
-use std::iter::FromIterator;
+use itertools::{assert_equal, enumerate};
 
 macro_rules! assert_panics {
     ($body:expr) => {
@@ -28,6 +25,7 @@ macro_rules! assert_panics {
 }
 
 #[test]
+#[cfg(feature = "std")]
 fn double_ended() {
     let a = ArcArray::linspace(0., 7., 8);
     let mut it = a.iter().cloned();
@@ -36,7 +34,7 @@ fn double_ended() {
     assert_eq!(it.next(), Some(1.));
     assert_eq!(it.rev().last(), Some(2.));
     assert_equal(aview1(&[1, 2, 3]), &[1, 2, 3]);
-    assert_equal(rev(aview1(&[1, 2, 3])), rev(&[1, 2, 3]));
+    assert_equal(aview1(&[1, 2, 3]).into_iter().rev(), [1, 2, 3].iter().rev());
 }
 
 #[test]
@@ -58,10 +56,11 @@ fn iter_size_hint() {
 }
 
 #[test]
+#[cfg(feature = "std")]
 fn indexed() {
     let a = ArcArray::linspace(0., 7., 8);
     for (i, elt) in a.indexed_iter() {
-        assert_eq!(i, *elt as Ix);
+        assert_eq!(i, *elt as usize);
     }
     let a = a.reshape((2, 4, 1));
     let (mut i, mut j, k) = (0, 0, 0);
@@ -76,21 +75,24 @@ fn indexed() {
     }
 }
 
-fn assert_slice_correct<A, S, D>(v: &ArrayBase<S, D>)
-where
-    S: Data<Elem = A>,
-    D: Dimension,
-    A: PartialEq + std::fmt::Debug,
-{
-    let slc = v.as_slice();
-    assert!(slc.is_some());
-    let slc = slc.unwrap();
-    assert_eq!(v.len(), slc.len());
-    assert_equal(v.iter(), slc);
-}
-
 #[test]
+#[cfg(feature = "std")]
 fn as_slice() {
+    use ndarray::Data;
+
+    fn assert_slice_correct<A, S, D>(v: &ArrayBase<S, D>)
+    where
+        S: Data<Elem = A>,
+        D: Dimension,
+        A: PartialEq + std::fmt::Debug,
+    {
+        let slc = v.as_slice();
+        assert!(slc.is_some());
+        let slc = slc.unwrap();
+        assert_eq!(v.len(), slc.len());
+        assert_equal(v.iter(), slc);
+    }
+
     let a = ArcArray::linspace(0., 7., 8);
     let a = a.reshape((2, 4, 1));
 
@@ -142,7 +144,7 @@ fn inner_iter() {
     //   [8, 9],
     //    ...
     assert_equal(
-        a.genrows(),
+        a.rows(),
         vec![
             aview1(&[0, 1]),
             aview1(&[2, 3]),
@@ -156,7 +158,7 @@ fn inner_iter() {
     b.swap_axes(0, 2);
     b.assign(&a);
     assert_equal(
-        b.genrows(),
+        b.rows(),
         vec![
             aview1(&[0, 1]),
             aview1(&[2, 3]),
@@ -171,13 +173,13 @@ fn inner_iter() {
 #[test]
 fn inner_iter_corner_cases() {
     let a0 = ArcArray::<i32, _>::zeros(());
-    assert_equal(a0.genrows(), vec![aview1(&[0])]);
+    assert_equal(a0.rows(), vec![aview1(&[0])]);
 
     let a2 = ArcArray::<i32, _>::zeros((0, 3));
-    assert_equal(a2.genrows(), vec![aview1(&[]); 0]);
+    assert_equal(a2.rows(), vec![aview1(&[]); 0]);
 
     let a2 = ArcArray::<i32, _>::zeros((3, 0));
-    assert_equal(a2.genrows(), vec![aview1(&[]); 3]);
+    assert_equal(a2.rows(), vec![aview1(&[]); 3]);
 }
 
 #[test]
@@ -185,7 +187,7 @@ fn inner_iter_size_hint() {
     // Check that the size hint is correctly computed
     let a = ArcArray::from_iter(0..24).reshape((2, 3, 4));
     let mut len = 6;
-    let mut it = a.genrows().into_iter();
+    let mut it = a.rows().into_iter();
     assert_eq!(it.len(), len);
     while len > 0 {
         it.next();
@@ -223,7 +225,7 @@ fn outer_iter() {
             found_rows.push(row);
         }
     }
-    assert_equal(a.genrows(), found_rows.clone());
+    assert_equal(a.rows(), found_rows.clone());
 
     let mut found_rows_rev = Vec::new();
     for sub in b.outer_iter().rev() {
@@ -251,7 +253,7 @@ fn outer_iter() {
         }
     }
     println!("{:#?}", found_rows);
-    assert_equal(a.genrows(), found_rows);
+    assert_equal(a.rows(), found_rows);
 }
 
 #[test]
@@ -305,7 +307,7 @@ fn axis_iter_zip() {
     let a = Array::from_iter(0..5);
     let iter = a.axis_iter(Axis(0));
     let mut b = Array::zeros(5);
-    Zip::from(&mut b).and(iter).apply(|b, a| *b = a[()]);
+    Zip::from(&mut b).and(iter).for_each(|b, a| *b = a[()]);
     assert_eq!(a, b);
 }
 
@@ -317,7 +319,7 @@ fn axis_iter_zip_partially_consumed() {
     while iter.next().is_some() {
         consumed += 1;
         let mut b = Array::zeros(a.len() - consumed);
-        Zip::from(&mut b).and(iter.clone()).apply(|b, a| *b = a[()]);
+        Zip::from(&mut b).and(iter.clone()).for_each(|b, a| *b = a[()]);
         assert_eq!(a.slice(s![consumed..]), b);
     }
 }
@@ -331,7 +333,7 @@ fn axis_iter_zip_partially_consumed_discontiguous() {
         consumed += 1;
         let mut b = Array::zeros((a.len() - consumed) * 2);
         b.slice_collapse(s![..;2]);
-        Zip::from(&mut b).and(iter.clone()).apply(|b, a| *b = a[()]);
+        Zip::from(&mut b).and(iter.clone()).for_each(|b, a| *b = a[()]);
         assert_eq!(a.slice(s![consumed..]), b);
     }
 }
@@ -370,7 +372,7 @@ fn outer_iter_mut() {
             found_rows.push(row);
         }
     }
-    assert_equal(a.genrows(), found_rows);
+    assert_equal(a.rows(), found_rows);
 }
 
 #[test]
@@ -484,7 +486,7 @@ fn axis_iter_mut_zip() {
     let mut cloned = orig.clone();
     let iter = cloned.axis_iter_mut(Axis(0));
     let mut b = Array::zeros(5);
-    Zip::from(&mut b).and(iter).apply(|b, mut a| {
+    Zip::from(&mut b).and(iter).for_each(|b, mut a| {
         a[()] += 1;
         *b = a[()];
     });
@@ -502,7 +504,7 @@ fn axis_iter_mut_zip_partially_consumed() {
             iter.next();
         }
         let mut b = Array::zeros(remaining);
-        Zip::from(&mut b).and(iter).apply(|b, a| *b = a[()]);
+        Zip::from(&mut b).and(iter).for_each(|b, a| *b = a[()]);
         assert_eq!(a.slice(s![consumed..]), b);
     }
 }
@@ -518,12 +520,13 @@ fn axis_iter_mut_zip_partially_consumed_discontiguous() {
         }
         let mut b = Array::zeros(remaining * 2);
         b.slice_collapse(s![..;2]);
-        Zip::from(&mut b).and(iter).apply(|b, a| *b = a[()]);
+        Zip::from(&mut b).and(iter).for_each(|b, a| *b = a[()]);
         assert_eq!(a.slice(s![consumed..]), b);
     }
 }
 
 #[test]
+#[cfg(feature = "std")]
 fn axis_chunks_iter_corner_cases() {
     // examples provided by @bluss in PR #65
     // these tests highlight corner cases of the axis_chunks_iter implementation
@@ -540,9 +543,9 @@ fn axis_chunks_iter_corner_cases() {
     assert_equal(
         it,
         vec![
-            arr2(&[[7.], [6.], [5.]]),
-            arr2(&[[4.], [3.], [2.]]),
-            arr2(&[[1.], [0.]]),
+            array![[7.], [6.], [5.]],
+            array![[4.], [3.], [2.]],
+            array![[1.], [0.]],
         ],
     );
 
@@ -747,8 +750,8 @@ fn iterators_are_send_sync() {
     _send_sync(&a.iter_mut());
     _send_sync(&a.indexed_iter());
     _send_sync(&a.indexed_iter_mut());
-    _send_sync(&a.genrows());
-    _send_sync(&a.genrows_mut());
+    _send_sync(&a.rows());
+    _send_sync(&a.rows_mut());
     _send_sync(&a.outer_iter());
     _send_sync(&a.outer_iter_mut());
     _send_sync(&a.axis_iter(Axis(1)));
